@@ -392,7 +392,7 @@ static int tda10045_fwupload(struct dvb_frontend* fe)
 
 	/* request the firmware, this will block until someone uploads it */
 	printk(KERN_INFO "tda1004x: waiting for firmware upload (%s)...\n", "/*(DEBLOBBED)*/");
-	ret = state->config->reject_firmware(fe, &fw, "/*(DEBLOBBED)*/");
+	ret = state->config->request_firmware(fe, &fw, "/*(DEBLOBBED)*/");
 	if (ret) {
 		printk(KERN_ERR "tda1004x: no firmware upload (timeout or file not found?)\n");
 		return ret;
@@ -529,13 +529,13 @@ static int tda10046_fwupload(struct dvb_frontend* fe)
 
 	/* eeprom firmware didn't work. Load one manually. */
 
-	if (state->config->reject_firmware != NULL) {
+	if (state->config->request_firmware != NULL) {
 		/* request the firmware, this will block until someone uploads it */
 		printk(KERN_INFO "tda1004x: waiting for firmware upload...\n");
-		ret = state->config->reject_firmware(fe, &fw, "/*(DEBLOBBED)*/");
+		ret = state->config->request_firmware(fe, &fw, "/*(DEBLOBBED)*/");
 		if (ret) {
 			/* remain compatible to old bug: try to load with tda10045 image name */
-			ret = state->config->reject_firmware(fe, &fw, "/*(DEBLOBBED)*/");
+			ret = state->config->request_firmware(fe, &fw, "/*(DEBLOBBED)*/");
 			if (ret) {
 				printk(KERN_ERR "tda1004x: no firmware upload (timeout or file not found?)\n");
 				return ret;
@@ -642,7 +642,7 @@ static int tda10046_init(struct dvb_frontend* fe)
 
 	if (tda10046_fwupload(fe)) {
 		printk("tda1004x: firmware upload failed\n");
-			return -EIO;
+		return -EIO;
 	}
 
 	// tda setup
@@ -895,9 +895,18 @@ static int tda1004x_get_fe(struct dvb_frontend *fe)
 {
 	struct dtv_frontend_properties *fe_params = &fe->dtv_property_cache;
 	struct tda1004x_state* state = fe->demodulator_priv;
+	int status;
 
 	dprintk("%s\n", __func__);
 
+	status = tda1004x_read_byte(state, TDA1004X_STATUS_CD);
+	if (status == -1)
+		return -EIO;
+
+	/* Only update the properties cache if device is locked */
+	if (!(status & 8))
+		return 0;
+
 	// inversion status
 	fe_params->inversion = INVERSION_OFF;
 	if (tda1004x_read_byte(state, TDA1004X_CONFC1) & 0x20)
@@ -997,7 +1006,8 @@ static int tda1004x_get_fe(struct dvb_frontend *fe)
 	return 0;
 }
 
-static int tda1004x_read_status(struct dvb_frontend* fe, fe_status_t * fe_status)
+static int tda1004x_read_status(struct dvb_frontend *fe,
+				enum fe_status *fe_status)
 {
 	struct tda1004x_state* state = fe->demodulator_priv;
 	int status;
