@@ -258,7 +258,7 @@ static int nfcwilink_download_fw(struct nfcwilink *drv)
 
 	rc = reject_firmware(&fw, file_name, &drv->pdev->dev);
 	if (rc) {
-		nfc_err(&drv->pdev->dev, "reject_firmware failed %d\n", rc);
+		nfc_err(&drv->pdev->dev, "request_firmware failed %d\n", rc);
 
 		/* if the file is not found, don't exit with failure */
 		if (rc == -ENOENT)
@@ -272,7 +272,7 @@ static int nfcwilink_download_fw(struct nfcwilink *drv)
 
 	if ((len == 0) || (ptr == NULL)) {
 		dev_dbg(&drv->pdev->dev,
-			"reject_firmware returned size %d\n", len);
+			"request_firmware returned size %d\n", len);
 		goto release_fw;
 	}
 
@@ -497,7 +497,7 @@ static struct nci_ops nfcwilink_ops = {
 
 static int nfcwilink_probe(struct platform_device *pdev)
 {
-	static struct nfcwilink *drv;
+	struct nfcwilink *drv;
 	int rc;
 	__u32 protocols;
 
@@ -566,7 +566,6 @@ static struct platform_driver nfcwilink_driver = {
 	.remove = nfcwilink_remove,
 	.driver = {
 		.name = "nfcwilink",
-		.owner = THIS_MODULE,
 	},
 };
 
