@@ -866,7 +866,7 @@ static int moxa_init_board(struct moxa_board_conf *brd, struct device *dev)
 
 	ret = reject_firmware(&fw, file, dev);
 	if (ret) {
-		printk(KERN_ERR "MOXA: reject_firmware failed. Make sure "
+		printk(KERN_ERR "MOXA: request_firmware failed. Make sure "
 				"you've placed '%s' file into your firmware "
 				"loader directory (e.g. /lib/firmware)\n",
 				file);
@@ -1094,7 +1094,7 @@ static int __init moxa_init(void)
 				continue;
 			}
 
-			printk(KERN_INFO "MOXA isa board found at 0x%.8lu and "
+			printk(KERN_INFO "MOXA isa board found at 0x%.8lx and "
 					"ready (%u ports, firmware loaded)\n",
 					baseaddr[i], brd->numPorts);
 
